<?php
	/**
	* SmartTemplateParser Class
	* Used by SmartTemplate Class
	*
	* @desc Used by SmartTemplate Class
	* @author Philipp v. Criegern philipp@criegern.com
	* @author Manuel 'EndelWar' Dalla Lana endelwar@aregar.it
	* @version 1.2.1 03.07.2006
	*
	* CVS ID: $Id: class.smarttemplateparser.php,v 1.9 2006/07/02 22:17:15 endelwar Exp $
	*/
/*
+--------------------------------------------------------------------------
|   Mega File Hosting Script v1.5 by 0m1d
|   ========================================
|   by Stephen Yabziz
|   (c) 2005-2006 YABSoft Services
|   http://www.yabsoft.com
|   ========================================
|   Web: http://www.P30vel.ir
|   Email: ywyhnchina@163.com
+--------------------------------------------------------------------------
|
|   > This class has been modified by Stephen Yabziz for YABSoft product
|   > The 'SmartTemplate' class is released under LGPL 2.1, which grant to you the right to use this library in proprietary software.
|   > And this modifed 'SmartTemplate' class is also released under LGPL 2.1!
|   > Last updated: 20th May 2008
+--------------------------------------------------------------------------
*/
	class SmartTemplateParser
	{
		/**
		* The template itself
		*
		* @access private
		*/
		var $template;

		/**
		* The template filename used to extract the dirname for subtemplates
		*
		* @access private
		*/
		var $template_dir;

        /**
		* Directory where php script which all templates need are stored
		*
		* @access private
		*/
		var $templatephp_dir;
  
        /**
		* Directory where extensions script which all templates need are stored
		*
		* @access private
		*/
		var $templateext_dir;
  
		/**
		* List of used SmartTemplate Extensions
		*
		* @access private
		*/
		var $extension_tagged  =  array();

        /**
		* Extension Prefix
		*
		* @access private
		*/
		var $extension_prefix  =  'mfhs';
  
		/**
		* Error messages
		*
		* @access public
		*/
		var $error;
  
         /**
		* Whether to use php code or php including
		*
		* @access private
		*/
		var $use_php;
  
        /**
		* Whether to store compiled php code or not (for debug purpose)
		*
		* @access private
		*/
		var $reuse_code;
  
        /**
		* Whether to use strict mode to parse the block name
        * the option need to be turned off to be compatiable with phpBB2 or phpBB3 engine
        * especially added for YABSoft product!
		*
		* @access private
		*/
		var $strict_mode = false;

		/**
		* SmartTemplateParser Constructor
		*
		* @param string $template_filename HTML Template Filename
		*/
		function SmartTemplateParser ( $template_filename )
		{
			// Load Template
			if ($hd = @fopen($template_filename, "r"))
			{
				if (filesize($template_filename))
				{
					$this->template = fread($hd, filesize($template_filename));
				}
				else
				{
					$this->template = 'SmartTemplate Parser Error[Line:'.__LINE__.']: File size is zero byte: "'.$template_filename.'"';
				}
				fclose($hd);
				// Extract the name of the template directory
				$this->template_dir = dirname($template_filename);
			}
			else
			{
				$this->template = 'SmartTemplate Parser Error[Line:'.__LINE__.']: File not found: "'.$template_filename.'"';
			}
		}

		/**
		* Main Template Parser
		*
		* @param string $compiled_template_filename Compiled Template Filename
		* @desc Creates Compiled PHP Template
		*/
		function compile( $compiled_template_filename = '' )
		{
			if (empty($this->template))
			{
				return false;
			}
			/* Quick hack to allow subtemplates */
			if(eregi("<!-- INCLUDE", $this->template))
			{
				preg_match_all('/<!-- INCLUDE ([a-zA-Z0-9_.]+) -->/', $this->template, $tvar);
				foreach($tvar[1] as $subfile)
				{
					if(file_exists($this->template_dir . $subfile))
					{
                        // a bad method to included multiple level sub templates?!
						//$subst = implode('',file($this->template_dir . "/$subfile"));
                        $subst = "<"."?php \$this->_tpl_include('$subfile');?".">";
					}
					else
					{
						$subst = 'SmartTemplate Parser Error: Subtemplate not found: \'' . $this->template_dir . $subfile . '\'';
					}
					$this->template = str_replace("<!-- INCLUDE $subfile -->", $subst, $this->template);
				}
			}
   
            /* Inclulding PHP script file */
			if($this->use_php&&eregi("<!-- INCLUDEPHP", $this->template))
			{
				preg_match_all('/<!-- INCLUDEPHP ([a-zA-Z0-9_.]+) -->/', $this->template, $tvar);
				foreach($tvar[1] as $subfile)
				{
					if(file_exists($this->templatephp_dir . $subfile))
					{
                        $subst = "<"."?php require '{$this->templatephp_dir}$subfile';?".">";
					}
					else
					{
						$subst = 'SmartTemplate Parser Error[Line:'.__LINE__.']: PHP script for template not found: \''.$this->templatephp_dir.$subfile.'\'';
					}
					$this->template = str_replace("<!-- INCLUDEPHP $subfile -->", $subst, $this->template);
				}
			}
   
            /* Inclulding PHP code section */
			if(eregi("<!-- PHP -->", $this->template))
			{
                if($this->use_php)
                {
                    preg_match_all('/<!-- PHP -->(.*?)<!-- ENDPHP -->/s', $this->template, $tvar);
				    foreach($tvar[1] as $phpcode)
				    {
					    $this->template = preg_replace("/<!-- PHP -->(.*?)<!-- ENDPHP -->/s", "<"."?php\n$phpcode\n?".">", $this->template);
				    }
                }
                else
                {
                    $this->template = preg_replace("/<!-- PHP -->(.*?)<!-- ENDPHP -->/s", "<!-- PHPCODE -->", $this->template);
                }
			}

			// END, ELSE Blocks
			$page  =  preg_replace("/<!-- ENDIF.+?-->/",  "<"."?php }?".">",  $this->template);
            $page  =  preg_replace("/<!-- ELSE[ ]* -->/",  "<"."?php } else { ?".">",  $page);

            // If stick mode is disabled, match all end blocks!
            if(!$this->strict_mode) preg_match_all('/<!-- END ([a-zA-Z0-9_.]+) -->/', $page, $var0);

            // ENDIF ENDXXX
            $page  =  preg_replace("/<!-- END[ a-zA-Z0-9_.]* -->/",  "<"."?php } --\$_stack_cnt;} ?".">",  $page);

			//	'BEGIN - END' Blocks
			if (preg_match_all('/<!-- BEGIN ([a-zA-Z0-9_.]+) -->/', $page, $var))
			{
                // If stick mode is disabled, appending parent blockname if necessary
                if(!$this->strict_mode)
                {
                    $beginblocks = $var[1];
                    $endblocks   = $var0[1];
                    $blocktree   = $temp = array();
                    foreach($endblocks as $k1=>$endblock)
                    {
                        foreach($beginblocks as $k2=>$beginblock)
                        {
                            if($endblock!=$beginblock)
                            {
                                $temp[$endblock][] = $beginblock;
                            }
                            // We will stop at the first accurence
                            // of the $endblock in $beginblocks!
                            else
                            {
                                if(is_array($temp))
                                {
                                    $blocktree[$k2]=$temp[$endblock];
                                    $temp = array();
                                }
                                // This $endblock level is removed
                                unset($beginblocks[$k2]);
                                break;
                            }
                        }
                    }
                }
                // Generateing code Replacing block statement
				foreach ($var[1] as $num=>$tag)
				{
                    $tag2 = $tag;
                    if(!$this->strict_mode)
                    {
                        // Ignore the block name: example.subrow
                        if(strpos($tag,'.')===false&&is_array($blocktree[$num]))
                        {
                            $tag = implode('.',$blocktree[$num]) . '.' . $tag;
                            unset($blocktree[$num]);
                        }
                    }
					list($parent, $block)  =  $this->var_name($tag);
					$code  =  "<"."?php\n"
							. "if (!empty(\$$parent"."['$block'])){\n"
							. "if (!is_array(\$$parent"."['$block']))"
							. "\$$parent"."['$block']=array(array('$block'=>\$$parent"."['$block']));\n"
							. "\$_tmp_arr_keys=array_keys(\$$parent"."['$block']);\n"
							. "if (\$_tmp_arr_keys[0]!='0')"
							. "\$$parent"."['$block']=array(0=>\$$parent"."['$block']);\n"
							. "\$_stack[\$_stack_cnt++]=\$$parent"."['$block'];\n"
							. "foreach (\$$parent"."['$block'] as \$$block"."_rowcnt=>\$$block) {\n"
							. "\$$parent"."['$block'][\$$block"."_rowcnt]['ROWCNT']=\$$block"."_rowcnt+1;\n"
							. "\$$parent"."['$block'][\$$block"."_rowcnt]['ALTROW']=(\$$block"."_rowcnt+1)%2;\n"
							. "\$$parent"."['$block'][\$$block"."_rowcnt]['ROWBIT']=(\$$block"."_rowcnt+1)%2;\n"
							. "?".">";
                    $tagstr   =  '<!-- BEGIN '.$tag2.' -->';
                    $tagstart =  strpos($page, $tagstr);
					$page  =  substr($page, 0, $tagstart) . $code . substr($page, $tagstart+strlen($tagstr));
				}
			}

            //	'IF nnn=mmm' Blocks
			if (preg_match_all('/<!-- (ELSE)?IF (.*?) -->/', $page, $var))
			{
				foreach ($var[2] as $cnt => $tag)
				{
                    $else  =  ($var[1][$cnt] == 'ELSE') ? '} else' : '';
                    $code  =  "<"."?php ".$this->compile_tag_if($tag,$else)." ?".">";
                    $page  =  str_replace($var[0][$cnt],  $code,  $page);
				}
			}
   
            //	'DEFINE varname="value"' Blocks
			if (preg_match_all('/<!-- DEFINE ([a-zA-Z0-9_.]+)\s*=\s*"(.*?)"\s* -->/', $page, $vars))
            {
                // Generateing code Replacing block statement
				foreach ($vars[1] as $num=>$var)
				{
                    list($parent, $block)  =  $this->var_name($var);

                    $parent = str_replace('_obj','this->data',$parent);

                    $code = "<"."?php "."\$$parent"."['$block'] = \"".$vars[2][$num].'"'." ?".">";

                    $page  =  str_replace($vars[0][$num],  $code,  $page);
				}
			}

			//	Replace Scalars
			if (preg_match_all('/{([a-zA-Z0-9_. >]+)}/', $page, $var))
			{
				foreach ($var[1] as $fulltag)
				{
					//	Determin Command (echo / $obj[n]=)
					list($cmd, $tag)  =  $this->cmd_name($fulltag);

					list($block, $skalar)  =  $this->var_name($tag);
					if(substr($skalar,0,2)=='L_')
                    {
                        $code  =  "<"."?php $cmd isset(\$$block"."['$skalar'])?\$$block"."['$skalar']:'<b>['.$skalar.']</b>';?".">";
                    }
                    else
                    {
                        $code  =  "<"."?php $cmd \$$block"."['$skalar'];?".">";
                    }

                    $page  =  str_replace('{'.$fulltag.'}'."\r\n",  $code."\r\n\r\n",  $page);
                    $page  =  str_replace('{'.$fulltag.'}'."\n",  $code."\n\n",  $page);
                    $page  =  str_replace('{'.$fulltag.'}',  $code,  $page);
				}
			}


			//	ROSI Special: Replace Translations
			/*
            if (preg_match_all('/<"([a-zA-Z0-9_.]+)">/', $page, $var))
			{
				foreach ($var[1] as $tag)
				{
					list($block, $skalar)  =  $this->var_name($tag);
					$code  =  "<"."?php\necho gettext('$skalar');\n?".">\n";
					$page  =  str_replace('<"'.$tag.'">',  $code,  $page);
				}
			}
            */

			//	Include Extensions
			$header = '';
			if (preg_match_all('/{PLUGIN\_([a-zA-Z0-9_]+):([^}]*)}/', $page, $var))
			{
				foreach ($var[2] as $cnt => $tag)
				{
					//	Determin Command (echo / $obj[n]=)
					list($cmd, $tag)  =  $this->cmd_name($tag);

					$extension  =  $var[1][$cnt];
					if (!isset($this->extension_tagged[$extension])&&is_file($this->templateext_dir . $this->extension_prefix . '_' . $extension .'.php'))
					{
						$header  .=  'include_once "' . $this->templateext_dir . $this->extension_prefix . '_' . $extension .'.php";'."\n";
						$this->extension_tagged[$extension]  =  true;
					}
					if (!strlen($tag))
					{
						$code  =  "<"."?php\n$cmd {$this->extension_prefix}_$extension();\n?".">\n";
					}
					elseif (substr($tag, 0, 1) == '"')
					{
						$code  =  "<"."?php\n$cmd {$this->extension_prefix}_$extension($tag);\n?".">\n";
					}
					elseif (strpos($tag, ','))
					{
						list($tag, $addparam)  =  explode(',', $tag, 2);
						list($block, $skalar)  =  $this->var_name($tag);
						if (preg_match('/^([a-zA-Z_]+)/', $addparam, $match))
						{
							$nexttag   =  $match[1];
							list($nextblock, $nextskalar)  =  $this->var_name($nexttag);
							$addparam  =  substr($addparam, strlen($nexttag));
							$code  =  "<"."?php\n$cmd {$this->extension_prefix}_$extension(\$$block"."['$skalar'],\$$nextblock"."['$nextskalar']"."$addparam);\n?".">\n";
						}
						else
						{
							$code  =  "<"."?php\n$cmd {$this->extension_prefix}_$extension(\$$block"."['$skalar'],$addparam);\n?".">\n";
						}
					}
					else
					{
						list($block, $skalar)  =  $this->var_name($tag);
						$code  =  "<"."?php\n$cmd {$this->extension_prefix}_$extension(\$$block"."['$skalar']);\n?".">\n";
					}
					$page  =  str_replace($var[0][$cnt],  $code,  $page);
				}
			}

			//	Add Include Header
			if (isset($header) && !empty($header))
			{
				$page  =  "<"."?php\n$header?".">$page";
			}

            //  Store complied code
            $this->compiled_content = $page;

			//	Store Code to Temp Dir
			if ($this->reuse_code&&strlen($compiled_template_filename))
			{
		        if ($hd  =  @fopen($compiled_template_filename,  "w"))
		        {
			        @fwrite($hd,  $page);
			        @fclose($hd);
			        return true;
			    }
			    else
			    {
			    	$this->error  =  'SmartTemplate Parser Error[Line:'.__LINE__.']Could not write compiled file.';
			        return true;
			    }
			}
			else
			{
				return true;
			}
		}
		/**
	    * Compile IF tags - much of this is from Smarty with
	    * some adaptions for our block level methods
	    * @private
	    */
		function compile_tag_if($tag_args, $elseif)
		{
			// Tokenize args for 'if' tag.
			preg_match_all('/(?:
						"[^"\\\\]*(?:\\\\.[^"\\\\]*)*"         |
						\'[^\'\\\\]*(?:\\\\.[^\'\\\\]*)*\'     |
						[(),]                                  |
						[^\s(),]+)/x', $tag_args, $match);

			$tokens = $match[0];
			$is_arg_stack = array();

			for ($i = 0, $size = sizeof($tokens); $i < $size; $i++)
			{
				$token = &$tokens[$i];

				switch ($token)
				{
					case '!==':
					case '===':
					case '<<':
					case '>>':
					case '|':
					case '^':
					case '&':
					case '~':
					case ')':
					case ',':
					case '+':
					case '-':
					case '*':
					case '/':
					case '@':
						break;

					case '==':
					case 'eq':
						$token = '==';
						break;

					case '!=':
					case '<>':
					case 'ne':
					case 'neq':
						$token = '!=';
						break;

					case '<':
					case 'lt':
						$token = '<';
						break;

					case '<=':
					case 'le':
					case 'lte':
						$token = '<=';
						break;

					case '>':
					case 'gt':
						$token = '>';
						break;

					case '>=':
					case 'ge':
					case 'gte':
						$token = '>=';
						break;

					case '&&':
					case 'and':
						$token = '&&';
						break;

					case '||':
					case 'or':
						$token = '||';
						break;

					case '!':
					case 'not':
						$token = '!';
						break;

					case '%':
					case 'mod':
						$token = '%';
						break;

					case '(':
						array_push($is_arg_stack, $i);
						break;

					case 'is':
						$is_arg_start = ($tokens[$i-1] == ')') ? array_pop($is_arg_stack) : $i-1;
						$is_arg	= implode('	', array_slice($tokens,	$is_arg_start, $i -	$is_arg_start));

						$new_tokens	= parse_is_expr($is_arg, array_slice($tokens, $i+1));

						array_splice($tokens, $is_arg_start, sizeof($tokens), $new_tokens);

						$i = $is_arg_start;

						// no break

					default:
						if (preg_match('#^((?:[a-z0-9\-_]+\.)+)?(\$)?(?=[a-zA-Z])([a-zA-Z0-9\-_]+)#s', $token, $varrefs))
						{
                            list($var_parent, $var_name) = ($this->var_name($token));
                            $token = "\$$var_parent"."['$var_name']";
						}

						break;
				}
			}
			//print_R( $tokens);
			return (($elseif) ? '} else if (' : 'if (') . (implode(' ', $tokens) . ') { ');
		}
		/**
	    * parse expression
	    * @private
	    */
		function _parse_is_expr($is_arg, $tokens)
		{
			$expr_end =	0;
			$negate_expr = false;

			if (($first_token = array_shift($tokens)) == 'not')
			{
				$negate_expr = true;
				$expr_type = array_shift($tokens);
			}
			else
			{
				$expr_type = $first_token;
			}

			switch ($expr_type)
			{
				case 'even':
					if (@$tokens[$expr_end] == 'by')
					{
						$expr_end++;
						$expr_arg =	$tokens[$expr_end++];
						$expr =	"!(($is_arg	/ $expr_arg) % $expr_arg)";
					}
					else
					{
						$expr =	"!($is_arg % 2)";
					}
					break;

				case 'odd':
					if (@$tokens[$expr_end] == 'by')
					{
						$expr_end++;
						$expr_arg =	$tokens[$expr_end++];
						$expr =	"(($is_arg / $expr_arg)	% $expr_arg)";
					}
					else
					{
						$expr =	"($is_arg %	2)";
					}
					break;

				case 'div':
					if (@$tokens[$expr_end] == 'by')
					{
						$expr_end++;
						$expr_arg =	$tokens[$expr_end++];
						$expr =	"!($is_arg % $expr_arg)";
					}
					break;

				default:
					break;
			}

			if ($negate_expr)
			{
				$expr =	"!($expr)";
			}

			array_splice($tokens, 0, $expr_end,	$expr);

			return $tokens;
		}

		/**
		* Splits Template-Style Variable Names into an Array-Name/Key-Name Components
		* {example}               :  array( "_obj",                   "example" )  ->  $_obj['example']
		* {example.value}         :  array( "_obj['example']",        "value" )    ->  $_obj['example']['value']
		* {example.0.value}       :  array( "_obj['example'][0]",     "value" )    ->  $_obj['example'][0]['value']
		* {top.example}           :  array( "_stack[0]",              "example" )  ->  $_stack[0]['example']
		* {parent.example}        :  array( "_stack[$_stack_cnt-1]",  "example" )  ->  $_stack[$_stack_cnt-1]['example']
		* {parent.parent.example} :  array( "_stack[$_stack_cnt-2]",  "example" )  ->  $_stack[$_stack_cnt-2]['example']
		*
		* @param string $tag Variale Name used in Template
		* @return array  Array Name, Key Name
		* @access private
		* @desc Splits Template-Style Variable Names into an Array-Name/Key-Name Components
		*/
		function var_name($tag)
		{
			$parent_level  =  0;
			while (substr($tag, 0, 7) == 'parent.')
			{
				$tag  =  substr($tag, 7);
				$parent_level++;
			}
			if (substr($tag, 0, 4) == 'top.')
			{
				$obj  =  '_stack[0]';
				$tag  =  substr($tag,4);
			}
			elseif ($parent_level)
			{
				$obj  =  '_stack[$_stack_cnt-'.$parent_level.']';
			}
			else
			{
				$obj  =  '_obj';
			}
			while (is_int(strpos($tag, '.')))
			{
				list($parent, $tag)  =  explode('.', $tag, 2);
				if (is_numeric($parent))
				{
					$obj  .=  "[" . $parent . "]";
				}
				else
				{
					$obj  .=  "['" . $parent . "'][\${$parent}_rowcnt]";
				}
			}
			$ret = array($obj, $tag);
			return $ret;
		}


		/**
		* Determine Template Command from Variable Name
		* {variable}             :  array( "echo",              "variable" )  ->  echo $_obj['variable']
		* {variable > new_name}  :  array( "_obj['new_name']=", "variable" )  ->  $_obj['new_name']= $_obj['variable']
		*
		* @param string $tag Variale Name used in Template
		* @return array  Array Command, Variable
		* @access private
		* @desc Determine Template Command from Variable Name
		*/
		function cmd_name($tag)
		{
			if (preg_match('/^(.+) > ([a-zA-Z0-9_.]+)$/', $tag, $tagvar))
			{
				$tag  =  $tagvar[1];
				list($newblock, $newskalar)  =  $this->var_name($tagvar[2]);
				$cmd  =  "\$$newblock"."['$newskalar']=";
			}
			else
			{
				$cmd  =  "echo";
			}
			$ret = array($cmd, $tag);
			return $ret;
		}

		/**
		* @return int Number of subtemplate included
		* @access private
		* @desc Count number of subtemplates included in current template
		*/
		function count_subtemplates()
		{
			preg_match_all('/<!-- INCLUDE ([a-zA-Z0-9_.]+) -->/', $this->template, $tvar);
			$count_subtemplates = count($tvar[1]);
			$ret = intval($count_subtemplates);
			return $ret;
		}
	}
?>
